@@ -26,6 +26,7 @@ from threading import Thread
 import threading
 import json
 import urllib3
+import ast
 
 
 from avmesos.client import MesosClient
@@ -87,6 +88,7 @@ class AirflowMesosScheduler(MesosClient):
         self.mesos_slave_docker_image = conf.get("mesos", "DOCKER_IMAGE_SLAVE")
         self.mesos_docker_volume_driver = conf.get("mesos", "DOCKER_VOLUME_DRIVER")
         self.mesos_docker_volume_dag_name = conf.get("mesos", "DOCKER_VOLUME_DAG_NAME")
+        self.mesos_docker_environment = conf.get("mesos", "DOCKER_ENVIRONMENT", fallback="")
         self.mesos_docker_volume_dag_container_path = conf.get(
             "mesos", "DOCKER_VOLUME_DAG_CONTAINER_PATH"
         )
@@ -104,14 +106,24 @@ class AirflowMesosScheduler(MesosClient):
             conf.get("mesos", "COMMAND_SHELL", fallback=True)
         ).lower()
 
+    def resource_heartbeat(self, heartbeat):
+        """If we got a heartbeat, run checks"""
+        self.log.info("Heartbeat")
+        if self.task_queue.empty():
+            self.log.debug("Suppress Mesos Framework")
+            self.driver.suppress()
+        else:
+            self.log.debug("Revive Mesos Framework")
+            self.driver.revive()
+       
     def resource_offers(self, offers):
         """If we got a offer, run a queued task"""
-        for i, offer in enumerate(offers):
-            if i == 0:
-                self.run_job(offer)
-            else:
-              offer.decline()
-            i += 1
+        while (not self.task_queue.empty()):
+            for i, offer in enumerate(offers):
+                if not self.run_job(offer):
+                     offertmp = offer.get_offer()
+                     self.log.info("Declined Offer: %s", offertmp["id"]["value"])
+                     offer.decline()        
 
     # pylint: disable=too-many-branches
     def run_job(self, mesos_offer):
@@ -127,9 +139,11 @@ class AirflowMesosScheduler(MesosClient):
 
         for resource in offer["resources"]:
             if resource["name"] == "cpus":
-                offer_cpus += resource["scalar"]["value"]
+                offer_cpus = resource["scalar"]["value"]
+                self.log.info("CPU's %d", offer_cpus)
             elif resource["name"] == "mem":
-                offer_mem += resource["scalar"]["value"]
+                offer_mem = resource["scalar"]["value"]
+                self.log.info("MEM %d", offer_mem)
 
         self.log.debug(
             "Received offer %s with cpus: %s and mem: %s",
@@ -142,16 +156,13 @@ class AirflowMesosScheduler(MesosClient):
         remaining_mem = offer_mem
 
         if remaining_cpus < self.task_cpu:
-            self.log.info("Offered CPU's are not enough: %d, %d", remaining_cpus, self.task_cpu)
+            self.log.info("Offered CPU's are not enough: %d, %d", remaining_cpus, self.task_cpu, offer["id"]["value"])
+            return False
         if remaining_mem < self.task_mem:
-            self.log.info("Offered MEM's are not enough: %d, %d", remaining_mem, self.task_mem)
+            self.log.info("Offered MEM's are not enough: %d, %d", remaining_mem, self.task_mem, offer["id"]["value"])
+            return False
         
-        while (
-            (not self.task_queue.empty())
-            and remaining_cpus >= self.task_cpu
-            and remaining_mem >= self.task_mem
-        ):
-
+        if (not self.task_queue.empty()):
             key, cmd, executor_config = self.task_queue.get()
             tid = self.task_counter
             self.task_counter += 1
@@ -203,8 +214,10 @@ class AirflowMesosScheduler(MesosClient):
 
             self.log.info("Launching task %d using offer %s", tid, offer["id"]["value"])
 
+            name = key.dag_id + "_" + key.task_id + "_" + str(key.execution_date.date()) + ":" + str(key.execution_date.time())
+
             task = {
-                "name": "AirflowTask %d" % tid,
+                "name": name,
                 "task_id": {"value": airflow_task_id},
                 "agent_id": {"value": offer["agent_id"]["value"]},
                 "resources": [
@@ -280,6 +293,15 @@ class AirflowMesosScheduler(MesosClient):
                 },
             }
 
+            # concat custom docker environment variables if they are configured
+            if len(self.mesos_docker_environment) > 0:
+                dockerEnv = ast.literal_eval(self.mesos_docker_environment)
+                lenTask = len(dockerEnv)
+                i = 0
+                while i < lenTask:
+                    task["command"]["environment"]["variables"].append(dockerEnv[i])
+                    i += 1
+
             # if the container would be UCR, we can attach tty
             if container_type == "MESOS":
                 task["container"]["tty_info"] = {
@@ -294,10 +316,12 @@ class AirflowMesosScheduler(MesosClient):
             tasks.append(task)
             remaining_cpus -= self.task_cpu
             remaining_mem -= self.task_mem
-        if len(tasks) > 0:
+        if len(tasks) > 0:            
             mesos_offer.accept(tasks, option)
+            return True
         else:
             mesos_offer.decline()
+            return False
 
     @provide_session
     def subscribed(self, driver, session=None):
@@ -472,6 +496,8 @@ class MesosExecutor(BaseExecutor):
         self.client.on(MesosClient.SUBSCRIBED, driver.subscribed)
         self.client.on(MesosClient.UPDATE, driver.status_update)
         self.client.on(MesosClient.OFFERS, driver.resource_offers)
+        self.client.on(MesosClient.HEARTBEAT, driver.resource_heartbeat)
+        
 
         self.mesos_framework = MesosExecutor.MesosFramework(self.client)
         self.mesos_framework.start()
@@ -511,10 +537,8 @@ class MesosExecutor(BaseExecutor):
                 return
             if state == "success":
                 self.log.info("tasks successfull %s", key)
-                self.task_queue.task_done()
             if state == "failed":
                 self.log.info("tasks failed %s", key)
-                self.task_queue.task_done()
             self.change_state(*results)
 
     def execute_async(
